/*  Rasteroids 0.01 - a free asteroids clone
    Copyright (C) 2000  Brian Luczkiewicz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <allegro.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include "display.h"
#include "ship.h"
#include "roids.h"
#include "bullets.h"
#include "rules.h"
#include "enemy.h"
#include "fire.h"
#include "game.h"
#include "skin.h"


/* Playing field bitmaps. Every refresh, I copy the background to
 * The arena to clear. The statusbar is an area for scores, title
 * image, etc...
 */
BITMAP * field;
BITMAP * field_bg;
BITMAP * screen_img;
BITMAP * status;

DATAFILE * fontDat;
PALETTE masterpal;




// Wrapper for the REAL findfile...
char * findfile(char * s)
{
	char * retval;
	retval = _findfile(s);
	//fprintf(stderr, "Findfile was given %s returning %s.\n",s,retval);
	s = retval;
	return s;
}

/* take a screenshot */
void takeshot()
{
    PALETTE pal;
    get_palette(pal);
    save_bmp("shot.bmp",screen,pal);
}

void display_init()
{

    if ((fontDat = load_datafile(findfile("font.dat"))) == NULL) {
	   fprintf(stderr," Error loading font.dat.");
	   abnormal_exit(2);
    }
    font = fontDat[0].dat;

    set_color_depth(24);

    if (set_gfx_mode(GFX_AUTODETECT,SCREEN_WIDTH,SCREEN_HEIGHT,0,0) < 0) {
      fprintf(stderr, "Aborting. Error initializing graphics");
      exit(-1);
    }
    screen_img = create_bitmap(SCREEN_WIDTH,SCREEN_HEIGHT);
    field = create_sub_bitmap(screen_img,0,STATUS_HEIGHT,FIELD_WIDTH,FIELD_HEIGHT);
    field_bg = create_bitmap(FIELD_WIDTH,FIELD_HEIGHT);
    status = create_sub_bitmap(screen_img,0,0,SCREEN_WIDTH,STATUS_HEIGHT);

    clear(status);

    if (!field || !field_bg) {
      fprintf(stderr, "Aborting. Error initializing graphics");
      exit(-1);
	}

    clear(screen_img);
    clear(field_bg);

	rect(field_bg,0,0,FIELD_WIDTH - 1, FIELD_HEIGHT - 1,makecol(255,255,255));
    blit(field_bg,field,0,0,0,0,FIELD_WIDTH,FIELD_HEIGHT);
    fade_in(masterpal,5);
}

void display_shutdown()
{
	fade_out(5);
}

void display_update()
{
    /* Get the background cleared */
    blit(field_bg,field,0,0,0,0,FIELD_WIDTH,FIELD_HEIGHT);
    ship_draw();
    enemy_draw();
    roids_draw();
    bullets_draw();
    rules_draw();
    fire_draw();

    vsync();
    blit(screen_img,screen,0,0,0,0,SCREEN_WIDTH,SCREEN_HEIGHT);
    clear(status);
}
