/*  Rasteroids 0.01 - a free asteroids clone
    Copyright (C) 2000  Brian Luczkiewicz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <allegro.h>
#include <stdio.h>

#include "rules.h"
#include "game.h"
#include "roids.h"
#include "ship.h"
#include "enemy.h"
#include "input.h"
#include "display.h"
#include "collis.h"
#include "fire.h"

#define MS(hz) 1000/hz

int end_game;
int pauseGame;
int mini;

void abnormal_exit (int code)
{
	allegro_exit();
	fprintf(stderr,"Rasteroids is exiting abnormally with error code %i\n",code);
	exit(code);
}

void run_once()
{
    game_init();
    game_run();
    game_shutdown();
}

volatile int sc=0;

void inc_speed_count()
{
	sc++;
}
END_OF_FUNCTION(inc_speed_count);

void game_init()
{
    display_init();
    input_init();
    enemy_init();
    rules_init();
    ship_init();
    roids_init();
    collis_init();
    bullets_init();
    fire_init();

	// increment timer at 60fps. 
	install_int_ex(inc_speed_count,BPS_TO_TIMER(60));

    pauseGame = 0;
    end_game = 0;
}

void game_run()
{
	while (end_game == 0) {
		
	    // If timer hasn't been incremented, skip the frame
		if (sc > 0) {
    		input_update();
	   	 	ship_update();		
	   	 	roids_update();
			rules_update();
 	  		display_update();
 	    	input_update();
 	    	fire_update();
 	    	bullets_update();
 	    	enemy_update();
 	    	collis_update();
    		rest(MS(fps)); /* use + and - keys to change speed at run time */
    		sc=0;
		}
	}
	textout_centre(screen,font,"Game Over",SCREEN_WIDTH/2,SCREEN_HEIGHT/2,139);
    rest(20);
   while(!key[KEY_ENTER] && !key[KEY_ESC]);
}

void game_miniframe()	/* this is one frame that doesn't update the ship. */
{                       /* used when the player dies */
    ship.safe = 1;
    fire_update();
    roids_update();
    enemy_update();
    display_update();
    rules_update();
    input_update();
    bullets_update();
    collis_update();
   // rest(MS(fps)); /* use + and - keys to change speed at run time */
    ship.safe = 0;
}

void game_shutdown()
{
    input_shutdown();
    enemy_shutdown();
    ship_shutdown();
    roids_shutdown();
    display_shutdown();
    bullets_shutdown();
    rules_shutdown();
    fire_shutdown();
    collis_shutdown();
}
