/*  Rasteroids 0.01 - a free asteroids clone
    Copyright (C) 2000  Brian Luczkiewicz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <allegro.h>
#include <math.h>
#include <stdio.h>
#include <time.h>

#include "roids.h"
#include "display.h"
#include "input.h"
#include "collis.h"
#include "rules.h"
#include "ship.h"
#include "game.h"

int numRoids;
struct roid_t roids[40];
BITMAP * roidSprite;

void roids_init()
{
 	int c;
 	roidSprite = create_bitmap(64,64);
    	if ((roidSprite = load_bmp(findfile("roid.bmp"),NULL)) == NULL) {
	   fprintf(stderr," Error loading roid.bmp.");
	   abnormal_exit(2);
	}

	/* start with 3 visible roids */
	numRoids = wave+2;
    if (numRoids > 10) numRoids = 10;

    for (c = 0; c < numRoids; c++) {
       	roids[c].vis  = 1;
        roids[c].Vel = 0;
        roids[c].size = 3;
    	roids[c].xPos = rand()%FIELD_WIDTH;
        roids[c].yPos = rand()%FIELD_HEIGHT;
        roids[c].ang  = rand()%255;
        srand((unsigned)clock());
	}
}

void roids_update()
{
	int c;
    int xVel,yVel,angDeg;

   	for (c = 0; c < numRoids; c++)
       if (roids[c].vis == 1) {

            /* randomly change course (slightly) +/- 22.5 deg */
         	if(rand()%100 == 1)
        		roids[c].ang += (rand()%64) - 32;

            /*get angle in range...for some reason, I have no luck with mod */
   			while (roids[c].ang > 255)
     			roids[c].ang -= 256;
   			while (roids[c].ang < 0)
     			roids[c].ang += 256;

   			angDeg = roids[c].ang * 360 / 256;

            /* correct roid stats in case they're dead */
			switch(roids[c].size) {
              case 1:
                   roids[c].Vel = 70;
                   roids[c].xSize = roids[c].ySize = 16;
                   break;
              case 2:
                   roids[c].Vel = 60;
                   roids[c].xSize = roids[c].ySize = 32;
                   break;
              case 3:
                   roids[c].Vel = 50;
                   roids[c].xSize = roids[c].ySize = 64;
                   break;
			}

   			/* Get x and y velocities based on angle and velocity.
   			* Thanks to visiken for the theory and formulas
   		 	*/
   			xVel = roids[c].Vel * cos(angDeg * (3.141593/180));
   			yVel = roids[c].Vel * sin(angDeg * (3.141593/180));

    		/* set the roid positions */
   			roids[c].xPos += xVel/10;
   			roids[c].yPos += yVel/10;

   			/* Handle screen wrap */
   			roids[c].xPos = (roids[c].xPos >= 640) ? 0 : roids[c].xPos;
   			roids[c].xPos = (roids[c].xPos < 0) ? 640 : roids[c].xPos;

   			roids[c].yPos = (roids[c].yPos >= 440) ? 0 : roids[c].yPos;
   			roids[c].yPos = (roids[c].yPos < 0) ? 440 : roids[c].yPos;

            srand((int) malloc(4));
            if (roids[c].safe > 0) roids[c].safe--;
   		}
}

void roid_break(int c,int safeTime)
{
 	roids[c].size--;
    if (roids[c].size > 0) {
        roids[c].safe = safeTime;
    	roids[numRoids] = roids[c];
    	roids[numRoids].ang -= 92;
		numRoids++;
        rules_action(ROIDSPLIT);
	} else {
		numRoids--;
        roids[c] = roids[numRoids];
        rules_action(ROIDKILL);
	}
}

void roids_shutdown()
{
}

void roids_draw()
{
	int c;
   for (c = 0;c < numRoids  && c < 40; c++) {
   	if (roids[c].vis == 1)
    	  switch(roids[c].size) {
      	  case 1:
               rotate_scaled_sprite(field,roidSprite,roids[c].xPos,roids[c].yPos,itofix(roids[c].ang),ftofix(0.25));
         		break;
           case 2:
               rotate_scaled_sprite(field,roidSprite,roids[c].xPos,roids[c].yPos,itofix(roids[c].ang),ftofix(0.5));
               break;
           case 3:
               rotate_sprite(field,roidSprite,roids[c].xPos,roids[c].yPos,itofix(roids[c].ang));
               break;
           default:
               break;
         }
	}
}
