/*  Rasteroids 0.01 - a free asteroids clone
    Copyright (C) 2000  Brian Luczkiewicz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <allegro.h>
#include <stdio.h>

#include "ship.h"
#include "display.h"
#include "input.h"
#include "math.h"
#include "game.h"


struct ship_t ship;

BITMAP * burnSprite1;
BITMAP * burnSprite2;
BITMAP * normSprite;

int shieldPower;

void ship_init()
{

    	if ((normSprite = load_bmp(findfile("ship.bmp"),masterpal)) == NULL) {
	   fprintf(stderr," Error loading ship.bmp.");
	   abnormal_exit(2);
	}
    	if ((burnSprite1 = load_bmp(findfile("burnship1.bmp"),NULL)) == NULL) {
	   fprintf(stderr," Error loading burnship1.bmp.");
	   abnormal_exit(2);
	}
	if ((burnSprite2 = load_bmp(findfile("burnship2.bmp"),NULL)) == NULL) {
	   fprintf(stderr," Error loading burnship2.bmp.");
	   abnormal_exit(2);
	}
   
    set_palette(masterpal);

    ship.sprite = normSprite;

    shieldPower = 300;

	ship.xPos = 640/2;
    ship.yPos = 440/2;
    ship.Vel = 0;
    ship.ang = 64;
    ship.xSize = 48;
    ship.ySize = 48;
    ship.safe = 50;
}

void ship_update()
{
   int xVel,yVel,angDeg;

   ship.ang -= leftNum*8;
   ship.ang += rightNum*8;

   if (shieldPower < 300 && shield == 0)
     shieldPower += 6;

   if (shield == 1 && shieldPower > 0) {
     shieldPower -= 9;
     ship.safe = 5;
   }

   if (shieldPower < 0)
     shieldPower = 0;


   /*get angle in range...for some reason, I have no luck with mod */
   while (ship.ang > 255)
     ship.ang -= 256;
   while (ship.ang < 0)
     ship.ang += 256;

   ship.Vel += downNum*7;
   ship.Vel -= upNum*13;

   if (upNum > 0) {
     if (ship.sprite == burnSprite1)
     	ship.sprite = burnSprite2;
     else
       ship.sprite = burnSprite1;
   } else
     ship.sprite = normSprite;

   angDeg = ship.ang * 360 / 256;

   /* handle terminal velocity */
   if (ship.Vel > 90 || ship.Vel < -150) ship.Vel = (ship.Vel > 90) ? 90 : -150 ;

   /* handle ship decay */
   ship.Vel = (ship.Vel < 15 && ship.Vel > -15) ? (ship.Vel > 0) ? ship.Vel - 1: ship.Vel + 1: (ship.Vel > 0) ? ship.Vel - 4 : ship.Vel + 4;
   ship.Vel = (ship.Vel < 4 && ship.Vel > -4) ? 0 : ship.Vel;

   /* Get x and y velocities based on angle and velocity.
   	* Thanks to visiken for the theory and formulas
    */
   xVel = ship.Vel * cos(angDeg * (3.141593/180));
   yVel = ship.Vel * sin(angDeg * (3.141593/180));

    /* set the ship positions */
   ship.xPos += xVel/10;
   ship.yPos += yVel/10;

   /* Handle screen wrap */
   ship.xPos = (ship.xPos >= 640) ? 0 : ship.xPos;
   ship.xPos = (ship.xPos < 0) ? 640 : ship.xPos;

   ship.yPos = (ship.yPos >= 440) ? 0 : ship.yPos;
   ship.yPos = (ship.yPos < 0) ? 440 : ship.yPos;

   if (ship.safe > 0) ship.safe--;
   /* clear variables */
   leftNum = rightNum = upNum = downNum = 0;
}

void ship_shutdown()
{
}

void ship_draw()
{
   if (shield == 1 && shieldPower > 0)
      circle(field,ship.xPos+24,ship.yPos+24,24,makecol(0,0,255));
   rotate_sprite(field,ship.sprite,ship.xPos,ship.yPos,itofix(ship.ang + 192));
}


