/*
    This file is part of "UFO 2000" aka "X-COM: Gladiators"
                         http://ufo2000.sourceforge.net/
    Copyright (C) 2000-2001  Alexander Ivanov aka Sanami <sanami@chat.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef MAP_H
#define MAP_H
#include <assert.h>
#include "cell.h"
#include "pck.h"
#include "spk.h"
#include "terrapck.h"
#include "place.h"
#include "soldier.h"

enum Mode {MAP2D, MAP3D, MAN, WATCH, UNIT_INFO};

struct GEODATA {
	unsigned short x_size;  //Distance e/w in tens of tiles
	unsigned short y_size;  //Distance n/s in tens of tiles
	unsigned short z_size;  //Height of map (tiles)
	unsigned short ship[2]; //The types of each of the two ships
	unsigned short terrain; //Terrain set; refers to the MCD combo sets ---  A JUNGLE, FOREST, etc...
	unsigned char  map[36]; //The actual map data - refers to the number at the end of map name; i.e. urban12 would be number
	unsigned char  use_rmp[36]; //Tells it if it should use the RMPs of each square(if no, there will be no units in that square)
	unsigned char  ship1_y; //These coords are given in tiles
	unsigned char  ship1_x;
	unsigned char  ship2_y;
	unsigned char  ship2_x;
	unsigned char  load_game; //If this is on, then the game is supposed to use the data from the GAME_xx directory
};

class Map {
private:
	GEODATA gd;

	static PCK *cursor;
	static SPK *scanbord;
	static char *m_scang;
	static unsigned short *m_loftemp;
	static int m_loftemp_num;
	int m_terrain_num;
	int m_terrain_set;

	static int dir2ofs[8];
	static char ofs2dir[3][3];

	Cell ***m_cell;
	TerraPCK *m_terrain;
public:
	static PCK *floorob, *smoke;
	static void initpck();
	static int valid_GEODATA(GEODATA *gd);
	static void new_GEODATA(GEODATA *mapdata);

	int level, width, height;
	int sel_lev, sel_row, sel_col;
	int x, y;

	Map();
	//Map(char *fname);
	//Map(int w, int h);
	~Map();
	void create(int l, int w, int h);
	void destroy();

	//void loadgeodata(char *fname);
	void loadterrain(int tid);
	void loadmaps(unsigned char *_map);
	int loadmap(char *fname, int _x, int _y);

	void drawitem(int itype, int gx, int gy);
	void draw_cell_pck(int _x, int _y, int _type, int _seen);
	void draw();
	void draw2d();
	void svga2d();
	BITMAP *create_bitmap_of_map();

	void set();
	void move(int ofs_x, int ofs_y);
	void set_sel(int mx, int my);

	int ofs;

	void center(int lev, int col, int row);
	int center2d(int xx, int yy);

	void step();
	void smoker();
	int explode(int lev, int col, int row, int type, int range, int damage);
	void explocell(int lev, int col, int row, int damage);

	int stopWALK(int oz, int ox, int oy, int part);
	int stopDOOR(int oz, int ox, int oy, int part);
	int passable(int oz, int ox, int oy, int dir);

	int stopLOS(int oz, int ox, int oy, int part);
	int viewable(int oz, int ox, int oy, int dir);
	int viewable_further(int vz, int vx, int vy);

	int calc_visible_cells(int z, int x, int y, int dir, char *visicells, int *ez, int *ex, int *ey);
	void build_visi();
	void rebuild_visi(int z, int x, int y);
	void build_visi_cell(int lev, int col, int row);
	int stopLOS_level(int dx, int dy, int lev, int col, int row);
	int haveGROUND(int lev, int col, int row);
	void update_visible_cells(char *visible_cells);

	void clearvis();
	void clearseen();

	int scroll(int mx, int my);

	void build_lof_cell(int _z, int _x, int _y, unsigned short *lof_cell);
	void build_lof_cell_part(int _z, int _x, int _y, int _part, unsigned short *lof_cell);
	BITMAP *create_lof_bitmap(int lev, int col, int row);
	void show_lof_cell();
	int pass_lof_cell(int _z, int _x, int _y);
	int pass_lof_cell_part(int _z, int _x, int _y, int _part);

	int open_door(int z, int x, int y, int part);
	void destroy_cell_part(int lev, int col, int row, int _part);
	void damage_cell_part(int lev, int col, int row, int _part, int _type);
	void apply_hit(int _z, int _x, int _y, int _type);

	int findneibo(int &dz, int &dx, int &dy, int value);
	int setneibos(int oz, int ox, int oy, int value);
	int pathfind(int sz, int sx, int sy, int dz, int dx, int dy, char *way);
	void path_show(int _z, int _x, int _y, char *way, int waylen);
	void draw_path_from(Soldier *s);
	int walk_time(int z, int x, int y);

	void drawline(int z_s, int x_s, int y_s, int z_d, int x_d, int y_d);

	Place *find_item(Item *it, int &lev, int &col, int &row);
	int find_place_num(Place *pl, int lev, int col, int row);

	int inside(int z, int x, int y);
	int cell_inside(int z, int x, int y);
	int find_ground(int lev, int col, int row);

	//int savestate(char *txt);
	//int saveitems(char *txt);
	//int save(char *fname);
	//int load(char *fname);
	//void load(char *mapfname, TerraPCK *pck2);
	int eot_save(char *buf, int &buf_size);
	int saveitems(char *txt);

	inline MCD *mcd(int lev, int col, int row, int type) {
		assert( (lev >= 0) && (lev < level)  &&
				  (col >= 0) && (col < width*10) &&
				  (row >= 0) && (row < height*10) &&
				  (type >= 0) && (type < 4) );
		MCD *m = &m_terrain->m_mcd[  m_cell[lev][col][row].type[type]  ];
		if (m->Tile_Type != type)
			return &m_terrain->empty;
		return m;
		//return &m_terrain->m_mcd[  m_cell[lev][col][row].type[type]	];
	}

	inline Cell *cell(int lev, int col, int row) {
		assert( (lev >= 0) && (lev < level)  &&
				  (col >= 0) && (col < width*10) &&
				  (row >= 0) && (row < height*10) );

		return &m_cell[lev][col][row];
	}

	inline Place *place(int lev, int col, int row) {
		assert( (lev >= 0) && (lev < level)  &&
				  (col >= 0) && (col < width*10) &&
				  (row >= 0) && (row < height*10) );

		return &m_cell[lev][col][row].place;
	}

	inline Soldier *man(int lev, int col, int row) {
		assert( (lev >= 0) && (lev < level)  &&
				  (col >= 0) && (col < width*10) &&
				  (row >= 0) && (row < height*10) );

		return m_cell[lev][col][row].man;
	}

	inline Soldier* sel_man() {
		return m_cell[sel_lev][sel_col][sel_row].man;
	}


	inline void set_man(int lev, int col, int row, Soldier *man) {
		assert( (lev >= 0) && (lev < level)  &&
				  (col >= 0) && (col < width*10) &&
				  (row >= 0) && (row < height*10) );
		m_cell[lev][col][row].man = man;
	}

	inline int visible(int lev, int col, int row) { return m_cell[lev][col][row].m_visible; }
	inline void set_visible(int lev, int col, int row, int value) { m_cell[lev][col][row].m_visible = value; }

	inline int seen(int lev, int col, int row) { return m_cell[lev][col][row].m_seen; }
	inline void set_seen(int lev, int col, int row, int value) { m_cell[lev][col][row].m_seen = value; }

	inline int smog_state(int lev, int col, int row) { return m_cell[lev][col][row].m_smog_state; }
	inline void set_smog_state(int lev, int col, int row, int value) { m_cell[lev][col][row].m_smog_state = value; }
	inline void dec_smog_state(int lev, int col, int row) { m_cell[lev][col][row].m_smog_state--; }
	inline void inc_smog_state(int lev, int col, int row) { m_cell[lev][col][row].m_smog_state++; }

	inline int smog_time(int lev, int col, int row) { return m_cell[lev][col][row].m_smog_time; }
	inline void set_smog_time(int lev, int col, int row, int value) { m_cell[lev][col][row].m_smog_time = value; }
	inline void dec_smog_time(int lev, int col, int row) { m_cell[lev][col][row].m_smog_time--; }

	inline int pfval(int lev, int col, int row) { return m_cell[lev][col][row].m_pfval; }
	inline void set_pfval(int lev, int col, int row, int value) { m_cell[lev][col][row].m_pfval = value; }
	inline int isStairs(int lev, int col, int row) { return (mcd(lev, col, row, 3)->T_Level < -15); }


	inline void center(Soldier *s) {
		assert( s != NULL );
		sel_lev = s->z;
		center(s->z, s->x, s->y);
	}
	void unhide();
	Map(int l, int c, int r, int _ter, unsigned char *_map);
	void set_map_data(int c, int r, char ter);
	void save(char *fname);
	void load(char *fname);
};

#endif
