/*
    This file is part of "UFO 2000" aka "X-COM: Gladiators"
                         http://ufo2000.sourceforge.net/
    Copyright (C) 2000-2001  Alexander Ivanov aka Sanami <sanami@chat.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "ldw.h"
#include "video.h"
#include "pck.h"

unsigned char PCK::m_tbb[0xFFFF];
unsigned short PCK::m_tbs[0xFFF];
char PCK::m_fname[0x100];

PCK::PCK(char *pckfname) {
	imgnum = 0;
	m_bmp = NULL; m_bmpofs = NULL;
	loadpck(pckfname);
}

PCK::~PCK() {
	if (m_bmpofs != NULL) delete []m_bmpofs;
	if (m_bmp != NULL) {
		for (int i=0; i<imgnum; i++) {
			if (m_bmp[i] != NULL)
				destroy_bitmap(m_bmp[i]);
		}
		delete []m_bmp;
	}
}

int PCK::add_image(char *fname) {
   PALLETE pal;
   BITMAP *bmp = load_bitmap(fname, pal);
   assert( bmp != NULL );
	m_bmp[imgnum] = create_bitmap(32,48); clear(m_bmp[imgnum]);
	m_bmpofs[imgnum] = 2;//(48-bmp->h);// / 2;
	blit(bmp, m_bmp[imgnum], 0, 0, 1, 0, bmp->w, bmp->h);
	destroy_bitmap(bmp);	
	imgnum++;
	return imgnum-1;
}

void PCK::loadpck(char *pckfname) {
	int fh = open(pckfname, O_RDONLY | O_BINARY);
	assert( fh != -1 );
	int pckbuflen = read(fh, m_tbb, 0xFFFF);
	close(fh);

	strcpy(m_fname, pckfname);
	strcpy(strrchr(m_fname,'.')+1,"tab");
	fh = open(m_fname, O_RDONLY | O_BINARY);
	//assert( fh != -1 );

	if (fh == -1) {
		imgnum = 1;
		m_tbs[0] = 0;
	} else {
		imgnum = read(fh, (char*)m_tbs, 0xFFFF) >> 1;
		close(fh);
	}
	m_tbs[imgnum] = pckbuflen;

	m_bmpofs  = new int[MAXNUM];
	m_bmp = new BITMAP*[MAXNUM];
	int i;
	for (i=0; i<MAXNUM; i++) {
		m_bmpofs[i] = 0;
		m_bmp[i] = NULL;
	}
	for (i=0; i<imgnum; i++) {
		m_bmp[i] = create_bitmap(32,48);
		clear(m_bmp[i]);
		m_bmpofs[i] = m_tbb[m_tbs[i]];

		unsigned char *ofs = (unsigned char *)m_bmp[i]->dat;

		for(int j=m_tbs[i]+1; j<m_tbs[i+1]-1; j++) {
			unsigned char a = m_tbb[j];
			if (a != 0xfe) {
				*ofs = a;
				ofs++;
			} else {
				ofs += m_tbb[++j];
			}
		}
	}
}

void PCK::showpck(int num, int xx, int yy) {
	draw_sprite(screen2, m_bmp[num], xx, yy + m_bmpofs[num] - 6);
}

void PCK::showpck(int num) {
	showpck(num, 0, 0);
}

void PCK::drawpck(int num, BITMAP *dest, int y) {
	draw_sprite(dest, m_bmp[num], 0, y + m_bmpofs[num] - 6);
}

/*BITMAP *PCK::create_bitmap(int num) {
}*/
