/*
    This file is part of "UFO 2000" aka "X-COM: Gladiators"
                         http://ufo2000.sourceforge.net/
    Copyright (C) 2000-2001  Alexander Ivanov aka Sanami <sanami@chat.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef SOLDIER_H
#define SOLDIER_H
#include "pck.h"
#include "spk.h"
#include "bullet.h"
#include "item.h"
#include "place.h"

#define P_SHL_RIGHT 0
#define P_SHL_LEFT  1
#define P_ARM_RIGHT 2
#define P_ARM_LEFT  3
#define P_LEG_RIGHT 4
#define P_LEG_LEFT  5
#define P_BACK      6
#define P_BELT      7
#define P_MAP       8
#define NUMBER_OF_PLACES  9
#define P_ARMOURY   9

#define S_HANDOB    0
#define S_XCOM_0    1
#define S_XCOM_1    2
#define S_XCOM_2    3
#define S_XCOM_3    4
#define S_SECTOID   5
#define SKIN_NUMBER  6

struct MANDATA {
	short Rank;  // Rank  == -1 --> Not used
	short Base;  // Base  == -1 --> Transfer in progress
	short	Craft; // Craft == -1 --> Not on any craft
	short int OldCraft;
	unsigned short Missions;
	unsigned short Kills;
	unsigned short Recovery;  // The number of days before their injuries are gone.
	unsigned short DeathCost; // The number of points you lose when they die.
	char           Name[26];  // There are actually 26 bytes allocated for this, but only the first 23 are used.  The names can be up to 22 bytes.
	unsigned char  TimeUnits;
	unsigned char  Health;
	unsigned char  Stamina;
	unsigned char  Reactions;
	unsigned char  Strength;
	unsigned char  Firing;
	unsigned char  Throwing;
	unsigned char  Close; //Close combat accuracy
	unsigned char  PsiStrength;
	unsigned char  PsiSkill;
	unsigned char  Bravery; // Bravery = ( 11 - x ) * 10
	unsigned char  TimeUnitsImp;
	unsigned char  HealthImp;
	unsigned char  StaminaImp;
	unsigned char  ReactionsImp;
	unsigned char  StrengthImp;
	unsigned char  FiringImp;
	unsigned char  ThrowingImp;
	unsigned char  CloseImp;
	unsigned char  BraveryImp; // * 10
	unsigned char  ArmorType;  //0=none
	unsigned char  PsiImprove; //The psionic improvement over the course of the last month
	unsigned char  fPsiTraining; //0=not in training  1=in psi training
	unsigned char  Promoted; //After a combat, this is set to 1 if they were promoted, 0 if not.
	unsigned char  fFemale;
	unsigned char  Appearance;
};


struct UNITDATA {
	unsigned char  Picture; //The picture type used on the tactical view
	unsigned char  BigPicture; //The armour type they're wearing on the inventory screen.
	unsigned char  u3;
	unsigned char  u4;
	unsigned char  u5;
	unsigned char  u6;
	unsigned char  u7;
	unsigned char  u8;
	unsigned char  u9;
	unsigned char  u10;
	unsigned char  Facing; //The direction the unit is facing:  00=North, 01=North East, and so on up to 7.
	unsigned char  u12;
	unsigned char  CurTU; //The number of TUs the unit has currently
	unsigned char  CurHealth;
	unsigned char  CurStun; //The stun level: if this is higher than CurHealth, they're stunned.
	unsigned char  CurEnergy;
	unsigned char  CurReactions;
	unsigned char  CurStrength;
	unsigned char  CurFront;
	unsigned char  CurLeft;
	unsigned char  CurRight;
	unsigned char  CurRear;
	unsigned char  CurUnder;
	unsigned char  CurFAccuracy; //Firing Accuracy
	unsigned char  CurTAccuracy; //Throwing Accuracy
	unsigned char  MaxTU;
	unsigned char  MaxHealth;
	unsigned char  MaxEnergy;
	unsigned char  MaxStrength;
	unsigned char  MaxFront;
	unsigned char  MaxLeft;
	unsigned char  MaxRight;
	unsigned char  MaxRear;
	unsigned char  MaxUnder;
	unsigned char  MaxFA; // ?
	unsigned char  MaxTA; // ?
	unsigned char  u37;
	unsigned char  PsiSkill;
	unsigned char  ItemDie; //The item type which is created when the unit dies
	unsigned char  u40;
	unsigned char  SoldierNo; //The entry # in soldier.dat which this unit equals.  FF means they were created for this fight (they're alien, tank, or civilian).
	unsigned char  u41;
	unsigned char  Rank; //The soldier rank for humans/aliens or, on tanks, the body type (0=tank 1=hovertank)
	unsigned char  ARank;//The alien rank, for research purposes (if you stun them)
	unsigned char  u45;
	unsigned char  EnergyUse; //If it's set to 255 no energy is used, if it's 1 then half as much energy as TUs is used, and if it's anything else then as much energy as TUs is used.
	unsigned char  Backpack;  //In humans: 0=normal backpack 1=large backpack  In tanks: 'head' type--0=cannon, 1=rocket, 2=laser, and so on.
	unsigned char  MIAScore;  //The number of points lost if they're Missing in Action
	unsigned char  u49;
	unsigned char  UnitBottom; //The bottom (in pixels above ground, I think) of the unit.  Used for collisions with bullets AND in drawing the picture (they are shifted up this much).
	unsigned char  u51;
	unsigned char  UnitHeight; //The number of pixels tall (I think) the unit is, for same purpose as UnitBottom
	unsigned char  u53;
	unsigned char  ItemLeft;  //The item type in the left hand--used only on the tactical screen (not used in the inventory screen)
	unsigned char  ItemRight; //The item type in the right hand
	unsigned char  u56;
	unsigned char  u57;
	unsigned char  PsiStrength;
	unsigned char  Morale;
	unsigned char  u60;
	unsigned char  u61;
	unsigned char  u62;
	unsigned char  u63;
	unsigned char  HeadWound; //Number of fatal wounds in the head.
	unsigned char  TorsoWound;
	unsigned char  RArmWound; //Right arm wounds
	unsigned char  LArmWound;
	unsigned char  RLegWound;
	unsigned char  LLegWound;
	unsigned char  u70;
	unsigned char  u71;
	unsigned char  u72;
	unsigned char  u73;
	unsigned char  u74;
	unsigned char  u75;
	unsigned char  u76;
	unsigned char  u77;
	unsigned char  Missions; //Number of missions so far this mission ... !!?
	unsigned char  Kills;    //Number of kills so far this mission
	unsigned char  u80;
	unsigned char  u81;
	unsigned char  u82;
	unsigned char  u83;
	unsigned char  u84;
	unsigned char  u85;
	unsigned char  u86;
	char           Name[25]; //The unit name!!
	unsigned char  u112;
	unsigned char  u113;
	unsigned char  u114;
	unsigned char  u115;
	unsigned char  Gender; //used in the inventory picture
	unsigned char  SkinColor; //the 'Appearance' stat from SOLDIER.DAT
	unsigned char  AttachedWep;
	unsigned char  AttachedAmmo; //The number of rounds left in the AttachedWep
	unsigned char  Flags; //if (Flags&&2) the unit can fly, otherwise it can't.
	unsigned char  u118;
	unsigned char  u119; //skin_type;  armour_type
	unsigned char  u120; //MOVED; // already moved this turn
};

enum State {SIT=0, STAND, MARCH, DIE};


class Soldier {
	friend class Map;
	friend class Platoon;
	friend class Inventory;
	friend class Icon;
	friend class Editor;
public:
	MANDATA md;
	ITEMDATA id;
private:
	UNITDATA ud;

	char m_visible_cells[4 * 6*10 * 6*10]; // do sizeof
	static char *****m_bof;
	static PCK **m_pck;
	static SPK *m_spk[4][2][4];
	//static SPK *m_unibord;
	static BITMAP *m_unibord;
	static int dir2ofs[8];
	static char ofs2dir[3][3];

	Soldier *m_next, *m_prev;
	Bullet *m_bullet;
	Place **m_place;	//8 - for internal editor use

	int NID;
	int dir, phase;
	State state;
	int z, x, y;
	int skin_type;
	int MOVED;

	char way[100];
	int curway, waylen;

	int FIRE_num;
	int FIRE_z, FIRE_x, FIRE_y;

	int enemy_num;
	int enemy_z[100], enemy_x[100], enemy_y[100];
	int seen_enemy_num;
	int seen_enemy_z[100], seen_enemy_x[100], seen_enemy_y[100];

public:

	static void initpck();
	static void initbof();

	Soldier(int _NID);
	Soldier(int _NID, int _z, int _x, int _y);
	Soldier(int _NID, int _z, int _x, int _y, MANDATA *sdat, ITEMDATA *idat);
	~Soldier();

	void initialize();

	void process_MANDATA();
	void show_MANDATA(int gx, int gy, int gcol);
	void process_ITEMDATA();
	void build_ITEMDATA();
	void build_items_stats(char *buf, int &len);


	void restore();
	int move(int ISLOCAL);
	void calc_visible_cells();

	void draw();
	void draw_inventory();
	void draw_unibord(int gx, int gy);
	void draw_selector(int select_y);
	void showspk();
	void drawbar(int col1, int col2, int x2, int y2, int val, int valmax);
	void drawinfo(int x, int y);
	void draw_bullet_way();
	void draw_enemy_seen();
	int center_enemy_seen();

	void turnto(int destdir);
	void wayto(int dest_lev, int dest_col, int dest_row);
   void finish_march(int ISLOCAL);
	void break_march();
	void faceto(int dest_col, int dest_row);
	int dirto(int src_col, int src_row, int dest_col, int dest_row);
	int open_door();
	int change_pose();
	int prime_grenade(int iplace, int delay_time, int req_time);
	int unload_ammo(Item *it);
	int load_ammo(int iplace, Item *it);
	void die();

	void apply_accuracy(float &fi, float &te);
	void apply_throwing_accuracy(float &fi, float &te, int weight);
	int required(int pertime);
	int FAccuracy(int peraccur, int TWOHAND);
	int TAccuracy(int peraccur);

	void precise_aiming();
	void try_shoot();
	void shoot(int tlev, int tcol, int trow);
	int thru(int z0, int x0, int y0, float ro, float fi, float te, float zA, int iplace, int req_time);
	int beam(int _z0, int _x0, int _y0, float _fi, float _te, int iplace, int req_time);
	int fire(int _z0, int _x0, int _y0, float _fi, float _te, int iplace, int req_time);
	int punch(int z0, int x0, int y0, float fi, float te, int iplace, int req_time);
	int aimedthrow(int z0, int x0, int y0, float fi, float te, int iplace, int req_time);

	int check_for_hit(int _z, int _x, int _y);
	void apply_hit(int _z, int _x, int _y, int _type);
	void hit(int pierce);
	void explo_hit(int pierce); //silent

	int ismoving();
	inline int is_marching() { return (state == MARCH); }
	int havetime(int ntime);
	void spend_time(int tm);
	int walktime(int _dir);

	void unlink();
	Soldier *nextman();
	Soldier *prevman();
	inline Soldier *next() { return m_next; }
	inline Soldier *prev() { return m_prev; }
	inline void set_next(Soldier *s) { m_next = s; }
	inline void set_prev(Soldier *s) { m_prev = s; }
	inline Bullet *bullet() { return m_bullet; }

	inline void create_map_place() { m_place[P_MAP] = new Place(0, 152, 20*10, 3); } //as map
	inline void destroy_map_place() { delete m_place[P_MAP]; }

	int eot_save(char *txt);

	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	int calc_ammunition_cost();
	Item *item_under_mouse(int ipl);
	int calctime(int src, int dst);
	Item *select_item(int &pn);
	int deselect_item(Item *&it, int it_place, int &req_time);
	Place *find_item(Item *it, int &lev, int &col, int &row);
	int haveitem(Item *it);

	inline Item *lhand_item() { return m_place[P_ARM_LEFT]->item(); }
	inline Item *rhand_item() { return m_place[P_ARM_RIGHT]->item(); }

	inline Item *item(int ip) {
		assert( (ip>=0) && (ip<NUMBER_OF_PLACES) );
		return m_place[ip]->item();
	}

	inline Item *item(int ip, int ix, int iy) {
		assert( (ip>=0) && (ip<NUMBER_OF_PLACES) );
		return m_place[ip]->item(ix, iy);
	}

	inline Item *getitem(int ip, int ix, int iy) {
		assert( (ip>=0) && (ip<NUMBER_OF_PLACES) );
		return m_place[ip]->get(ix, iy);
	}

	inline int putitem(Item *it, int ip, int ix, int iy) {
		assert( (ip>=0) && (ip<NUMBER_OF_PLACES) );
		return m_place[ip]->put(it, ix, iy);
	}

	inline int putitem(Item *it, int ip) {
		assert( (ip>=0) && (ip<NUMBER_OF_PLACES) );
		return m_place[ip]->put(it);
	}

	int place(Place *place);
	inline Place *place(int ip) {
		assert( (ip>=0) && (ip<NUMBER_OF_PLACES) );
		return m_place[ip];
	}

	void destroy_all_items();
	void draw_deselect_times(int sel_item_place);
	void damage_items(int damage);
};

#endif
