/*
    This file is part of "UFO 2000" aka "X-COM: Gladiators"
                         http://ufo2000.sourceforge.net/
    Copyright (C) 2000-2001  Alexander Ivanov aka Sanami <sanami@chat.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <assert.h>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "ldw.h"
#include "video.h"
#include "terrapck.h"


TerraPCK::TerraPCK(char *pckfname) : PCK(pckfname) {
	m_mcdnum = 0;
	m_mcd = new MCD[MAXNUM];
	m_blackbmp = new BITMAP* [MAXNUM];
	for (int i=0; i<MAXNUM; i++) {
		m_blackbmp[i] = NULL;
		memset(&m_mcd[i], 0, sizeof(MCD));
	}
	assert(imgnum > 0);
	create_blackbmp(0, imgnum);
	strcpy(m_fname, pckfname);
	loadmcd(0, imgnum);
	memset(&empty, 0, sizeof(MCD));
	empty.No_Floor = 1;
}

TerraPCK::~TerraPCK() {
	delete []m_mcd;
	for (int i=0; i<MAXNUM; i++) {
		if (m_blackbmp[i] != NULL)
			destroy_bitmap(m_blackbmp[i]);
	}
	delete []m_blackbmp;
}

void TerraPCK::add(char *pckfname) {
	strcpy(m_fname, pckfname);

	int fh = open(m_fname, O_RDONLY | O_BINARY);
	assert( fh != -1 );
	int newlen = read(fh, m_tbb, 0xFFFF);
	assert(newlen > 0);
	close(fh);

	strcpy(strrchr(m_fname,'.')+1,"tab");
	fh = open(m_fname, O_RDONLY | O_BINARY);
	assert( fh != -1 );
	int newnum = read(fh, (char *)m_tbs, 0xFFF) >> 1;
	assert(newnum > 0);
	close(fh);
	m_tbs[newnum] = newlen;
#ifdef DEBUG
	cprintf(" newlen = %d newnum = %d", newlen, newnum);
#endif
	for (int num=0; num<newnum; num++) {
		m_bmp[imgnum+num] = create_bitmap(32,48);
		clear(m_bmp[imgnum+num]);
		m_bmpofs[imgnum+num] = m_tbb[m_tbs[num]];

		unsigned char *ofs = (unsigned char *)m_bmp[imgnum+num]->dat;
		for(int i=m_tbs[num]+1; i<m_tbs[num+1]-1; i++) {
			unsigned char a = m_tbb[i];
			if (a != 0xfe) {
				*ofs = a;
				ofs++;
			} else {
				ofs += m_tbb[++i];
			}
		}
	}
	create_blackbmp(imgnum, newnum);
	loadmcd(imgnum, newnum);
	imgnum += newnum;
}

void TerraPCK::loadmcd(int start, int size) {
	strcpy(strrchr(m_fname,'.')+1,"mcd");
	int fh = open(m_fname, O_RDONLY | O_BINARY);
	assert( fh != -1 );
	int v = read(fh, m_tbb, 0xFFFF);
	assert( v > 0 );
	for (int i=0; i<size; i++)
		memcpy(&m_mcd[start+i], m_tbb+i*62, 62);
	close(fh);

	m_mcdstart[m_mcdnum++] = start;
	m_mcdstart[m_mcdnum] = start+size;
}

void TerraPCK::create_blackbmp(int start, int size) {
	for (int num=start; num<start+size; num++) {
		m_blackbmp[num] = create_bitmap(32,48);
		clear(m_blackbmp[num]);
		for (int i=0; i<32*48; i++)
			if (((char*)m_bmp[num]->dat)[i] != 0)
				 ((char*)m_blackbmp[num]->dat)[i] = 15;
	}
}

void TerraPCK::showblackpck(int num, int xx, int yy) {
	draw_sprite(screen2, m_blackbmp[num], xx, yy + m_bmpofs[num] - 6);
}


int TerraPCK::mcdstart(int mcd_num) {
	for(int i=0; i<m_mcdnum; i++)
		if (mcd_num < m_mcdstart[i+1])
			return m_mcdstart[i];
	return -1;
}
