/*
    This file is part of "UFO 2000" aka "X-COM: Gladiators"
                         http://ufo2000.sourceforge.net/
    Copyright (C) 2000-2001  Alexander Ivanov aka Sanami <sanami@chat.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "ldw.h"
#include "video.h"
#ifdef WIN32
#include <allegro/aintern.h>
#include <allegro/aintwin.h>
#endif
#include "wind.h"
#include "../ufo2000.h"

unsigned long FLAGS=0;
extern Wind *info;

#ifdef DJGPP
BEGIN_COLOR_DEPTH_LIST
	COLOR_DEPTH_8
END_COLOR_DEPTH_LIST
/*
BEGIN_GFX_DRIVER_LIST
	GFX_DRIVER_VGA
	GFX_DRIVER_VESA1
END_GFX_DRIVER_LIST
*/
#endif

//PALLETE gamepal;
RGB *gamepal;
DATAFILE *datafile;

BITMAP *mouser, *selector;
BITMAP *screen2;
BITMAP *screen_text;

int SCREEN2W=320, SCREEN2H=200;

volatile int DRAWIT=0;
void drawit_timer() {
	DRAWIT++;
}
END_OF_FUNCTION(drawit_timer);

int REDRAW=0;
#ifdef WIN32
static void switch_in_callback() {
	REDRAW++;
	//directdraw->RestoreAllSurfaces();
	dd_prim_surface->Restore();
	//info->redraw_full();
}
#endif

void initvideo() {
	LOCK_VARIABLE(DRAWIT);
	LOCK_FUNCTION(drawit_timer);

	datafile = load_datafile("#");
	if (datafile == NULL) {
		datafile = load_datafile("ufo2000.dat");
		if (datafile == NULL) {
			allegro_exit();
			printf("Error loading datafile!\n\n");
			exit(1);
		}
	}
	//selector = load_bmp("selector.bmp", gamepal);
	selector = (BITMAP*)datafile[DAT_SELECT].dat;
	//mouser = load_bmp("mouse.bmp", gamepal);
	mouser = (BITMAP*)datafile[DAT_MOUSE].dat;

	screen2 = create_bitmap(SCREEN2W, SCREEN2H);
	clear(screen2);
	screen_text = create_bitmap(640, 400-10-SCREEN2H);
	clear(screen_text);

	initpal("geodata/palettes.dat");

	//font->height = 8;
	//font->dat.dat_8x8 = (FONT_8x8 *)(font8x8cyr+32*8);
	font = (FONT*)datafile[DAT_FONT_CYR].dat;
	create_small_font();
	create_large_font();

}

void change_screen_mode() {
	if (FLAGS & F_SWITCHVIDEO) {
		if (FLAGS & F_FULLSCREEN) {
			FLAGS &= ~F_FULLSCREEN;
		} else {
			FLAGS |= F_FULLSCREEN;
		}
		set_video_mode();
		reset_video();
      REDRAW = 1;
	}
}

void set_video_mode() {
	set_color_depth(8);
#ifdef DJGPP
	set_gfx_mode(GFX_AUTODETECT, 640, 400, 0, 0);
#elif WIN32
	if (FLAGS & F_FULLSCREEN)
		set_gfx_mode(GFX_DIRECTX, 640, 400, 0, 0);
	else
		set_gfx_mode(GFX_DIRECTX_WIN, 640, 400, 0, 0);
	
	set_display_switch_mode(SWITCH_AMNESIA);
	set_display_switch_callback(SWITCH_IN, switch_in_callback);
#else
	set_gfx_mode(GFX_AUTODETECT, 640, 400, 0, 0);
#endif
}

void reset_video() {
	set_palette(gamepal);
	position_mouse(160, 100);
	set_mouse_range(0, 0, SCREEN2W-1, SCREEN2H-1);
	set_mouse_sprite(mouser);
	set_mouse_speed(1, 1);
	gui_fg_color = 0;
	gui_bg_color = 1;
	REDRAW = 1;
}

char *palettes;
int palettes_size;

void initpal(char *fname) {
	int fh = open(fname, O_RDONLY | O_BINARY);
	assert( fh != -1 );
	palettes_size = filelength( fh );

	palettes = new char[palettes_size];
	palettes_size = read(fh, palettes, palettes_size);
	close(fh);

	gamepal = (RGB *)datafile[DAT_GAMEPAL].dat;
}

void setpal(RGB pal[PAL_SIZE], int pofs) {
	//RGB pal[PAL_SIZE];
	int j = pofs;//0xC18;

	for (int i=0; i<PAL_SIZE; i++) {
		pal[i].r = palettes[j++];
		pal[i].g = palettes[j++];
		pal[i].b = palettes[j++];
		//pal[i].filler = 0;
	}
	set_palette(pal);
}

void savescreen() {
	BITMAP *scr = create_bitmap(SCREEN_W, SCREEN_H);
	blit(screen, scr, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

	save_pcx("snapshot.pcx", scr, gamepal);
	destroy_bitmap(scr);
}

#define POLY 0x8408
//												  16	12	5
// this is the CCITT CRC 16 polynomial X  + X  + X  + 1.
// This works out to be 0x1021, but the way the algorithm works
// lets us use 0x8408 (the reverse of the bit pattern).  The high
// bit is always assumed to be set, thus we only use 16 bits to
// represent the 17 bit value.

unsigned short crc16(char *data_p) {
	unsigned char i;
	unsigned int data;
	unsigned int crc = 0xffff;
	unsigned short length = strlen(data_p);

	if (length == 0)
		return (~crc);
	do {
		for (i=0, data=(unsigned int)0xff & *data_p++; i < 8; i++, data >>= 1) {
			if ((crc & 0x0001) ^ (data & 0x0001))
				crc = (crc >> 1) ^ POLY;
			else
				crc >>= 1;
		}
	} while (--length);

	crc = ~crc;
	data = crc;
	crc = (crc << 8) | (data >> 8 & 0xff);

	return (crc);
}

#include "icon.h"
extern Icon *icon;
extern Wind *info;
extern BITMAP *info_background;
extern BITMAP *logo;

void resize_screen2(int vw, int vh) {
	if (((SCREEN2W<=320)&&(vw<0)) || ((SCREEN2H <=200)&&(vh<0)))
		return;

	SCREEN2W+=vw; SCREEN2H+=vh;
	if (SCREEN2W > 600) SCREEN2W = 600;
	if (SCREEN2H > 350) SCREEN2H = 350;

	destroy_bitmap(screen2);
	screen2 = create_bitmap(SCREEN2W, SCREEN2H);

	destroy_bitmap(screen_text);
	screen_text = create_bitmap(640, 400-10-SCREEN2H);
	clear(screen_text);

	clear(info_background);
	draw_sprite(info_background, logo, 640-logo->w, SCREEN2H);
	blit(info_background, screen, 0,0, 0,0, 640, 400);

   //info->resize(info_background, 0, SCREEN2H, 640, 390);
	delete(info);
	//info = new Wind(info_background, 0, SCREEN2H, 640, 390, 1, " Messages ");
	info = new Wind(info_background, 5, SCREEN2H+5, 630, 390-5, 1);
	//info = new Wind(0, 0, 79, (400-SCREEN2H)/8-2, " Messages ", 0x07, 1);

	icon->setxy((SCREEN2W-320)/2, SCREEN2H-56);
	set_mouse_range(0, 0, SCREEN2W-1, SCREEN2H-1);
	position_mouse(SCREEN2W/2, SCREEN2H/2);
	//clear(screen);
}

int askmenu(char *mess) {
	int omx = mouse_x, omy = mouse_y;
	set_mouse_range(0, 0, 639, 399);

	int sel = alert(mess, "", "", "OK", "Cancel", 0, 0);
	//set_mouse_speed(2, 2);
	position_mouse(omx, omy);
	//set_mouse_range(0, 0, 319, 199);
	//set_mouse_range(0, 0, SCREEN2W-1, SCREEN2H-1);
	set_mouse_range(0, 0, screen2->w-1, screen2->h-1);
	if (sel == 1)
		return 1;
	return 0;
}
