/* XOP source code (c) 2001 Christopher Emirzian
   All Rights Reserved

   Description: Joystick detection, Input functions
*/

#include <stdio.h>
#include <stdlib.h>
//#include <conio.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include <allegro.h>
#include "jgmod.h"
#include <png.h>

//#include "loadpng.h"
#include "main.h"
#include "system.h"
#include "thing.h"
#include "tmake.h"
#include "draw.h"
//#include "level.h"
#include "input.h"
//#include "shrap.h"
#include "play.h"
//#include "file.h"
//#include "unpack.h"
#include "menu.h"

int initJoystick(int detectType, int loadSaved)
{
  char msg[128];
  char xopjoy[16];
  int x;

  sprintf(xopjoy,"xopjoy.dat");
  #ifdef DJGPP
  sprintf(xopjoy,"xopjoyd.dat");
  #endif

  if(loadSaved==0)
    remove(xopjoy);

  remove_joystick();

  if(install_joystick(detectType)!=0)
     return 1;

  if(joy_type==JOY_TYPE_NONE)
     return 1;

  #ifdef DJGPP
  if(joy_type==JOY_TYPE_STANDARD) // attempt to detect 4 button
  {
    remove_joystick();
    if(install_joystick(JOY_TYPE_4BUTTON)!=0)
    {
      remove_joystick();
      install_joystick(detectType);
    }
  }
  #endif

  if(!access(xopjoy,0))
    load_joystick_data(xopjoy);
  else
  {
    clear_keybuf();

    for(x=0; x<num_joysticks; x++)
    {
      while(joy[x].flags & JOYFLAG_CALIBRATE)
      {
        sprintf(msg,"%s",calibrate_joystick_name(x));

        clear(screen);
        textout_centre(screen, font, "Joystick calibration", SCREEN_W/2, (SCREEN_H/2)-26, colWhite);
        textout_centre(screen, font, joystick_driver->name, SCREEN_W/2, (SCREEN_H/2)-16, colWhite);

        textout_centre(screen, font, msg, SCREEN_W/2, (SCREEN_H/2), colWhite);
        textout_centre(screen, font, "and press a key.", SCREEN_W/2, (SCREEN_H/2)+10, colWhite);

        if((readkey()&0xFF)==27)
          return 1;

        if(calibrate_joystick(x)!=0)
        {
          clear(screen);
          textout_centre(screen, font, "Error calibrating joystick!", SCREEN_W/2, SCREEN_H/2, colWhite);
          readkey();
          return 1;
        }
      }
    }

    save_joystick_data(xopjoy);
  }

  clear_keybuf();

  clear(screen);

  return 0;
}

int doJoyDetect(int detectType, int loadSaved)
{
  if(gOpt.noJoyDetect==1)
  {
    play.control=1;
    return 0;
  }

  if(initJoystick(detectType,loadSaved)==0)
  {
    if(num_joysticks==0)
      play.control=1;
    else
      play.control=2;
  }
  else
    play.control=1;

  return 0;
}

void updateInput()
{
  if(play.control==1)
  {
    pInput[INPUT_U]=key[play.keydef[KEY_UP01]];
    pInput[INPUT_D]=key[play.keydef[KEY_DOWN01]];
    pInput[INPUT_L]=key[play.keydef[KEY_LEFT01]];
    pInput[INPUT_R]=key[play.keydef[KEY_RIGHT01]];
    pInput[INPUT_A1]=key[play.keydef[KEY_ACTION01]];
    pInput[INPUT_A2]=key[play.keydef[KEY_ACTION02]];
    pInput[INPUT_A3]=key[play.keydef[KEY_ACTION03]];
    pInput[INPUT_A4]=key[play.keydef[KEY_ACTION04]];
  }

  if(play.control==2)
  {
    poll_joystick();

    pInput[INPUT_U]=joy[0].stick[0].axis[1].d1;
    pInput[INPUT_D]=joy[0].stick[0].axis[1].d2;
    pInput[INPUT_L]=joy[0].stick[0].axis[0].d1;
    pInput[INPUT_R]=joy[0].stick[0].axis[0].d2;
    pInput[INPUT_A1]=joy[0].button[play.joydef[JOY_ACTION01]].b;
    pInput[INPUT_A2]=joy[0].button[play.joydef[JOY_ACTION02]].b;
    pInput[INPUT_A3]=joy[0].button[play.joydef[JOY_ACTION03]].b;
    pInput[INPUT_A4]=joy[0].button[play.joydef[JOY_ACTION04]].b;

    if(pInput[INPUT_U]==0)
      pInput[INPUT_U]=key[play.keydef[KEY_UP01]];
    if(pInput[INPUT_D]==0)
      pInput[INPUT_D]=key[play.keydef[KEY_DOWN01]];
    if(pInput[INPUT_L]==0)
      pInput[INPUT_L]=key[play.keydef[KEY_LEFT01]];
    if(pInput[INPUT_R]==0)
      pInput[INPUT_R]=key[play.keydef[KEY_RIGHT01]];
    if(pInput[INPUT_A1]==0)
      pInput[INPUT_A1]=key[play.keydef[KEY_ACTION01]];
    if(pInput[INPUT_A2]==0)
      pInput[INPUT_A2]=key[play.keydef[KEY_ACTION02]];
    if(pInput[INPUT_A3]==0)
      pInput[INPUT_A3]=key[play.keydef[KEY_ACTION03]];
    if(pInput[INPUT_A4]==0)
      pInput[INPUT_A4]=key[play.keydef[KEY_ACTION04]];
  }

  if(pInput[INPUT_A1]==0)
    pInput[INPUT_A1]=key[KEY_ENTER];
}

void doInput()
{
  int x,oldPause;

  updateInput();

  if(gStat==1)
    for(x=0; x<8; x++)
      pInput[x]=0;

  if(player.stat!=-1 && player.dir==0 && pauseStat==0)
    doPlayerInput();

  if(player.dir==-1 && pauseStat==1) // this is to allow the continue countdown to work
    doPlayer();

  if(key[KEY_F12])
  {
    oldPause=pauseStat;
    screenShot();
    pauseStat=oldPause;
  }

  if(closeButton==1)
    doQuit();

  if(key[KEY_ESC])
  {
    while(key[KEY_ESC]) { }

    if(backDoor==0)
    {
      for(x=0; x<maxsound; x++)
        if(snd[x])
          stop_sample(snd[x]);

      mainMenuInit();
    }
    else
      doQuit();
  }

  if(key[KEY_P])
  {
    while(key[KEY_P]) { }

    if(pauseStat==0)
    {
      pauseStat=1;
      set_mod_volume(gOpt.musicVol/2);
    }
    else
    {
      pauseStat=0;
      set_mod_volume(gOpt.musicVol);
      resetCounter();
    }
  }

  if(key[KEY_M])
    displayMemory();
}