/* XOP source code (c) 2001 Christopher Emirzian
   All Rights Reserved

   Description: XOP system functions, misc. game functions
*/

#include <stdio.h>
#include <stdlib.h>
//#include <conio.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include <allegro.h>
#include "jgmod.h"
#include <png.h>

#include "loadpng.h"
#include "main.h"
#include "system.h"
#include "thing.h"
#include "tmake.h"
#include "draw.h"
#include "level.h"
#include "input.h"
#include "shrap.h"
#include "play.h"
#include "file.h"
#include "unpack.h"
#include "menu.h"
#include "circ.h"

void initSystem()
{
  int x;

  // init some stuff

  player.dir=0;
  dirtyScreen=0;
  closeButton=0;
  register_png_file_type();

  loadHiscore();

  // allocate memory for some stuff

  if((thing=(tObj *)malloc(maxthing*sizeof(tObj)))==NULL)
    bombOut(900,0);

  if((sh=(shrapObj *)malloc(maxshrap*sizeof(shrapObj)))==NULL)
    bombOut(900,0);

  if((actMain=(actObj *)malloc(maxact*sizeof(actObj)))==NULL)
    bombOut(900,0);

  // start counter

  LOCK_VARIABLE(sCount);
  LOCK_FUNCTION(sCountAdd);
  LOCK_FUNCTION(resetCounter);
  LOCK_FUNCTION(waitCounter);

  resetCounter();
  install_int_ex(sCountAdd, BPS_TO_TIMER(55));

  // set up screen

  xopfont=readPNG(getPath("png/system/xopfont.png"));
  cursor=readPNG(getPath("png/system/cursor.png"));
  borderBmp=readPNG(getPath("png/system/border.png"));

  drawGameBorder();
  drawText(screen,"Loading ...",sXoff+2,sYoff+2,64);

  // create transparency tables

  drawText(screen,"Creating color maps",sXoff+2,sYoff+10,64);

  create_rgb_table(&rgb_table, palGame, NULL);
  rgb_map=&rgb_table;
  create_trans_table(&trans_table, palGame, 128, 128, 128, NULL);
  create_color_table(&flash_table_white, palGame, giveFlashColor, NULL);
  create_color_table(&flash_table_red, palGame, giveFlashColorRed, NULL);
  set_trans_blender(0,0,0,128);

  // create bitmaps

  drawText(screen,"Creating bitmaps            ",sXoff+2,sYoff+10,64);

  buffer=create_bitmap(320,320);   clear(buffer);   set_clip(buffer,0,0,280,300);
  bgBuffer=create_bitmap(320,320); clear(bgBuffer); set_clip(bgBuffer,0,0,280,300);
  bgBuffer2=create_bitmap(320,320); clear(bgBuffer2); set_clip(bgBuffer2,0,0,280,300);
  bufStat=create_bitmap(278,24);   clear(bufStat);  // size of status : 278x18

  // load system/player graphics

  drawText(screen,"Loading sprites             ",sXoff+2,sYoff+10,64);

  sprPlay[1]=readPNG(getPath("png/game/pship01.png"));
  sprPlay[2]=readPNG(getPath("png/game/pship02.png"));
  sprPlay[3]=readPNG(getPath("png/game/pship03.png"));
  sprPlay[4]=readPNG(getPath("png/game/pship04.png"));
  sprPlay[5]=readPNG(getPath("png/game/pship05.png"));

  sprPlay[6]=readPNG(getPath("png/system/begin.png"));
  sprPlay[7]=readPNG(getPath("png/system/gameover.png"));
  sprPlay[8]=readPNG(getPath("png/system/comp.png"));
  sprPlay[9]=readPNG(getPath("png/system/pause.png"));
  sprPlay[10]=readPNG(getPath("png/system/pshipicon.png"));
  sprPlay[11]=readPNG(getPath("png/system/rsicon.png"));
  sprPlay[12]=readPNG(getPath("png/system/continue.png"));
  sprPlay[13]=readPNG(getPath("png/system/warning.png"));

  sprPlay[14]=readPNG(getPath("png/system/button01.png"));
  sprPlay[15]=readPNG(getPath("png/system/button02.png"));
  sprPlay[16]=readPNG(getPath("png/system/menu01b.png"));
  sprPlay[17]=readPNG(getPath("png/system/endlevel.png"));
  sprPlay[18]=readPNG(getPath("png/system/endlevel2.png"));

  sprPlay[19]=readPNG(getPath("png/system/menu01.png"));

  // load main graphics

  for(x=0; x<maxsprite; x++)
    sprMain[x]=NULL;
  for(x=0; x<maxsprite2; x++)
    sprID[x]=-1;

  for(x=0; x<maxtile; x++)
    tile[x]=NULL;
  for(x=0; x<maxtile; x++)
    tileStat[x].tile=TILE_EMPTY;

  color_map=&flash_table_white;
  set_trans_blender(0,0,0,255);
  loadImage(getPath("data/sprite.dat"),0,6);
  color_map=&trans_table;
  set_trans_blender(0,0,0,128);

  sprMainOff=0;
  for(x=0; x<maxsprite2; x++)
  {
    if(sprMain[x])
      sprMainOff++;
  }

  // load sounds

  drawText(screen,"Loading sounds             ",sXoff+2,sYoff+10,64);

  loadSound(getPath("data/sound.dat"),19);

  // load data

  loadGameData();
}

void loadGameData()
{
  int x;

  readActObj(getPath("data/main.act"),actMain,sprMain,1);

  for(x=0; x<maxdat2; x++)
  {
    thingDef[x].com[0]=0; thingAct[x].com[0]=0;
  }

  readDatFile(getPath("data/tdef.def"),thingDef,9);
  readDatFile(getPath("data/tact.def"),thingAct,9);
  readDatFile(getPath("data/shot.def"),shotDef,11);
  readDatFile(getPath("data/move.def"),moveDef,12);
  readDatFile(getPath("data/circle.def"),circleDef,14);
  readDatFile(getPath("data/exp.def"),expDef,15);
  readDatFile(getPath("data/link.def"),linkDef,16);
  readDatFile(getPath("data/shrap.def"),shrapDef,10);

  for(x=0; x<maxdat2; x++)
    if(thingDef[x].com[0]!=thingAct[x].com[0])
      bombOut(28,x);

  // load player stuff

  readActObj(getPath("data/play.act"),actPlay,sprPlay,0);
  readDatFile(getPath("data/play.def"),playDef,0);

  play.shotType=playDef[1].com[PLAYDEF_SHOTTYPE];
  play.shotType2=playDef[1].com[PLAYDEF_SHOTTYPE2];
  play.bombType=playDef[1].com[PLAYDEF_BOMBTYPE];
  play.rShotType=playDef[1].com[PLAYDEF_RSHOTTYPE];
  play.dalType=playDef[1].com[PLAYDEF_DALTYPE];
  play.dalSilvType=playDef[1].com[7];

  play.shotDamage=(float)(playDef[3].com[0])/100;
  play.shotTimerReload=playDef[3].com[1];

  play.bombDamage=(float)(playDef[3].com[2])/100;
  play.bombTimerReload=playDef[3].com[3];

  play.rShotDamage=(float)(playDef[3].com[4])/100;

  play.wpn[WEAPON_BOTS].type=playDef[4].com[0];
  play.wpn[WEAPON_BOTS].shotType=playDef[4].com[1];
  play.wpn[WEAPON_BOTS].wpnType=playDef[4].com[2];
  play.wpn[WEAPON_BOTS].damage=(float)(playDef[4].com[3])/100;
  play.wpn[WEAPON_BOTS].timerReload=playDef[4].com[4];
  play.wpn[WEAPON_BOTS].weight=(float)(playDef[4].com[5])/100;
  play.wpn[WEAPON_BOTS].dalUpgrade=playDef[4].com[6]*100;

  play.wpn[WEAPON_VELOCITY].type=playDef[5].com[0];
  play.wpn[WEAPON_VELOCITY].shotType=playDef[5].com[1];
  play.wpn[WEAPON_VELOCITY].wpnType=playDef[5].com[2];
  play.wpn[WEAPON_VELOCITY].damage=(float)(playDef[5].com[3])/100;
  play.wpn[WEAPON_VELOCITY].timerReload=playDef[5].com[4];
  play.wpn[WEAPON_VELOCITY].weight=(float)(playDef[5].com[5])/100;
  play.wpn[WEAPON_VELOCITY].dalUpgrade=playDef[5].com[6]*100;

  play.wpn[WEAPON_LASER].type=playDef[6].com[0];
  play.wpn[WEAPON_LASER].shotType=playDef[6].com[1];
  play.wpn[WEAPON_LASER].wpnType=playDef[6].com[2];
  play.wpn[WEAPON_LASER].damage=(float)(playDef[6].com[3])/100;
  play.wpn[WEAPON_LASER].timerReload=playDef[6].com[4];
  play.wpn[WEAPON_LASER].weight=(float)(playDef[6].com[5])/100;
  play.wpn[WEAPON_LASER].dalUpgrade=playDef[6].com[6]*100;

  play.wpn[WEAPON_MISSLES].type=playDef[7].com[0];
  play.wpn[WEAPON_MISSLES].shotType=playDef[7].com[1];
  play.wpn[WEAPON_MISSLES].wpnType=playDef[7].com[2];
  play.wpn[WEAPON_MISSLES].damage=(float)(playDef[7].com[3])/100;
  play.wpn[WEAPON_MISSLES].timerReload=playDef[7].com[4];
  play.wpn[WEAPON_MISSLES].weight=(float)(playDef[7].com[5])/100;
  play.wpn[WEAPON_MISSLES].dalUpgrade=playDef[7].com[6]*100;

  play.wpn[WEAPON_IMPLOSION].type=playDef[8].com[0];
  play.wpn[WEAPON_IMPLOSION].shotType=playDef[8].com[1];
  play.wpn[WEAPON_IMPLOSION].wpnType=playDef[8].com[2];
  play.wpn[WEAPON_IMPLOSION].damage=(float)(playDef[8].com[3])/1000;
  play.wpn[WEAPON_IMPLOSION].timerReload=playDef[8].com[4];
  play.wpn[WEAPON_IMPLOSION].weight=(float)(playDef[8].com[5])/100;
  play.wpn[WEAPON_IMPLOSION].dalUpgrade=playDef[8].com[6]*100;

  play.wpn[WEAPON_PLASMA].type=playDef[9].com[0];
  play.wpn[WEAPON_PLASMA].shotType=playDef[9].com[1];
  play.wpn[WEAPON_PLASMA].wpnType=playDef[9].com[2];
  play.wpn[WEAPON_PLASMA].damage=(float)(playDef[9].com[3])/1000;
  play.wpn[WEAPON_PLASMA].timerReload=playDef[9].com[4];
  play.wpn[WEAPON_PLASMA].weight=(float)(playDef[9].com[5])/100;
  play.wpn[WEAPON_PLASMA].dalUpgrade=playDef[9].com[6]*100;

  play.actlist[ACT_STILL]=playDef[2].com[ACT_STILL];
  play.actlist[ACT_LEFT]=playDef[2].com[ACT_LEFT];
  play.actlist[ACT_RIGHT]=playDef[2].com[ACT_RIGHT];
  play.actlist[ACT_UNLEFT]=playDef[2].com[ACT_UNLEFT];
  play.actlist[ACT_UNRIGHT]=playDef[2].com[ACT_UNRIGHT];
}

void initGame()
{
  int x;

  if(key[KEY_R])
    showRects=1;
  else
    showRects=0;

  // set level order

  gOpt.lComp=0;

  for(x=0; x<16; x++)
    gOpt.lOrder[x]=x;

  gOpt.lComp=gOpt.lSkip;

  if(key[KEY_F1])
    gOpt.lComp=0;
  if(key[KEY_F2])
    gOpt.lComp=1;
  if(key[KEY_F3])
    gOpt.lComp=2;
  if(key[KEY_F4])
    gOpt.lComp=3;
  if(key[KEY_F5])
    gOpt.lComp=4;
  if(key[KEY_F6])
    gOpt.lComp=5;
  if(key[KEY_F7])
    gOpt.lComp=6;
  if(key[KEY_F8])
    gOpt.lComp=7;
  if(key[KEY_F9])
    gOpt.lComp=8;

  // init player stuff

  play.spd1=(float)(playDef[0].com[PLAYDEF_SPEED1])/100;
  play.spd2=(float)(playDef[0].com[PLAYDEF_SPEED2])/100;

  play.spd1+=play.wpn[play.wpn1].weight+play.wpn[play.wpn2].weight;
  play.spd2+=play.wpn[play.wpn1].weight+play.wpn[play.wpn2].weight;

  for(x=0; x<8; x++)
  {
    play.wpn[x].dalUpCount=0;
    play.wpn[x].damageUp=0;
    if(powerup==0)
      play.wpn[x].level=1;
    else
      play.wpn[x].level=9;
  }

  circ[3].x=8; // left engine
  circ[3].y=28;
  circ[3].oColor=13;
  circ[3].fColor=15;
  circ[3].size=3;
  circ[3].life=-2;
  circ[3].vib=1;
  circ[3].pos=0;
  player.circ[0]=3;

  circ[1].x=15; // right engine
  circ[1].y=28;
  circ[1].oColor=13;
  circ[1].fColor=15;
  circ[1].size=3;
  circ[1].life=-2;
  circ[1].vib=1;
  circ[1].pos=0;
  player.circ[1]=1;

  circ[2].x=10; // shot 1
  circ[2].y=4;
  circ[2].oColor=3;
  circ[2].fColor=15;
  circ[2].size=6;
  circ[2].life=-1;
  circ[2].vib=1;
  circ[2].pos=0;
  player.circ[2]=2;

  circ[0].x=10; // shot 2
  circ[0].y=4;
  circ[0].oColor=3;
  circ[0].fColor=15;
  circ[0].size=6;
  circ[0].life=-1;
  circ[0].vib=1;
  circ[0].pos=0;
  player.circ[3]=0;

  play.score=0;
  play.dal=0;

  play.shotTimer=0;
  play.shotNum=0;
  play.shotMax=8;
  play.shotLevel=0;

  play.bombTimer=0;
  play.bombLevel=0;

  play.rShotLevel=0;

  play.refNum=3;
  if(invun==1)
    play.refNum=6000;

  play.sTimer=0;

  play.health=1;
  play.lives=3;

  if(gOpt.skillMode==0)
    play.lives++;

  gOpt.skillVar=gOpt.skillMode-2;

  if(gOpt.skillMode==5)
    gOpt.skillVar=6;

  if(gOpt.skillMode==6)
    gOpt.skillVar=12;

  if(gOpt.expMode==5)
    gOpt.expMode=10;

  shieldAdd=gOpt.skillVar/5;
}

void gameLoop()
{
  resetCounter();
  for(;;)
  {
    while(sCount>0)
    {
      doInput();        // doPlayerInput (moves player and activates firing) and misc. input (pause, esc, screenshot)

      if(pauseStat==0)
      {
        doBackground(); // scrolls bgs and activates bg scripts
        doTiles();      // animates bg tiles
        doTimer();      // counts down and activates timers for delayed scripts/explosions
        doMisc();       // misc stuff
        doPlayer();     // does player stuff; timers, hit detection w/ bg, continue/game over, reflect shields
        doThings();     // animates and controls all things; broken up into timers, movement, links and hit detect
        doCircles();    // animates and controls circles
        doShrap();      // animates and controls shrapnel
      }
      gCount++;

      if(sCount>7)      // prevents game logic from hogging the CPU
        resetCounter(); // the game will slow down instead of appearing to freeze

      waitCounter();
    }

    if(sCountDrawOk==1)
    {
      sortThings();     // sorts the player, shrapnel, bgs and things for use with drawScreen() 
      drawScreen();     // draws the game screen and status bar

      sCountDrawOk=0;
    }
  }
}

void doQuit()
{
  int x;

  remove("xop.tmp");

  closePackfile();

  if(buffer) destroy_bitmap(buffer);
  if(bgBuffer) destroy_bitmap(bgBuffer);
  if(bgBuffer2) destroy_bitmap(bgBuffer2);
  if(bufStat) destroy_bitmap(bufStat);
  if(cursor) destroy_bitmap(cursor);
  if(xopfont) destroy_bitmap(xopfont);
  if(borderBmp) destroy_bitmap(borderBmp);

  for(x=0; x<32; x++)
  {
    if(sprPlay[x])
      destroy_bitmap(sprPlay[x]);
  }
  for(x=0; x<maxsprite; x++)
  {
    if(sprMain[x])
      destroy_bitmap(sprMain[x]);
  }
  for(x=0; x<maxtile; x++)
  {
    if(tile[x])
      destroy_rle_sprite(tile[x]);
  }

  if(thing)
    free(thing);
  if(sh)
    free(sh);
  if(actMain)
    free(actMain);

  for(x=0; x<maxbg; x++)
  {
    free(bg[x].map);
    free(bg[x].script);
  }

  for(x=0; x<maxsound; x++)
  {
    if(snd[x])
      destroy_sample(snd[x]);
  }

  stopMOD();

  set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
  if(backDoor==0)
    allegro_message("XOP (c) 2001 Christopher Emirzian\nPublic demo 0.4\n6-x-2001");

  allegro_exit();

  exit(0);
}

void closeWindow()
{
  closeButton=1;
}

void doStdInput()
{
  updateInput();
  screenShot();

  if(closeButton==1)
    doQuit();
}

/*__inline*/ int calctX(float X, float Z)
{
  return (zdepth*(X-200))/((zdepth-Z))+200;
}

/*__inline*/ int calctY(float Y, float Z)
{
  return (zdepth*(Y-150))/((zdepth-Z))+150;
}

/*__inline*/ float calctXF(float X, float Z)
{
  return (zdepth*(X-200))/((zdepth-Z))+200;
}

/*__inline*/ float calctYF(float Y, float Z)
{
  return (zdepth*(Y-150))/((zdepth-Z))+150;
}

/*__inline*/ int hit(int x1, int y1, int z1, int x2, int y2, int z2, int xs1, int ys1, int zs1, int xs2, int ys2, int zs2)
{
  return(!(x1>x2+xs2 || x2>x1+xs1 || y1>y2+ys2 || y2>y1+ys1 || z1>z2+zs2 || z2>z1+zs1));
}

/*__inline*/ int hit2(int x1, int y1, int x2, int y2, int xs1, int ys1, int xs2, int ys2)
{
  return(!(x1>x2+xs2 || x2>x1+xs1 || y1>y2+ys2 || y2>y1+ys1));
}

int hitpp(BITMAP *sprite1, BITMAP *sprite2, int x1, int y1, int x2, int y2)
{
	int bx1,by1,bx2,by2,bx3,by3; // intersection points
	int x,y,lenx,leny;           // used for the detection loop

	// get intersection point

	bx1=x1-x2;
  if(bx1<0)
    bx1=0;

	by1=y1-y2;
  if(by1<0)
    by1=0;
	
	if(x1<x2)
    bx2=x2-x1;
  else
    bx2=0;

	if(y1<y2)
    by2=y2-y1;
  else
    by2=0;

  if(x1>x2+((BITMAP *)sprite2)->w-((BITMAP *)sprite1)->w)
	  bx3=x2+((BITMAP *)sprite2)->w-x1;
	else
	  bx3=((BITMAP *)sprite1)->w;

	if(y1>y2+((BITMAP *)sprite2)->h-((BITMAP *)sprite1)->h)
		by3=y2+((BITMAP *)sprite2)->h-y1;
	else
		by3=((BITMAP *)sprite1)->h;

	lenx=abs(bx2-bx3);
	leny=abs(by2-by3);
	
	// do the collision detect

	for(y=0; y<leny; y++)
	  for(x=0; x<lenx; x++)
		  if(sprite1->line[by2+y][bx2+x]!=colTrans)
				if(sprite2->line[by1+y][bx1+x]!=colTrans)
					return 1;

  // no collision

	return 0;
}

/*__inline*/ void rotPos(float x, float y, float t)
{
  float tCos;

  tCos=cos(t);
  t=sin(t);

  rx=(x*tCos)-(y*t);
  ry=(y*tCos)+(x*t);
}

/*__inline*/ int comp(const int *i, const int *j)
{
  return *i - *j;
}

/*__inline*/ int compL(const unsigned long *i, const unsigned long *j)
{
  return *j - *i;
}

void sCountAdd()
{
  sCount++;
}
END_OF_FUNCTION(sCountAdd);

void resetCounter()
{
  sCount=0;
  sCountDrawOk=0;
}
END_OF_FUNCTION(resetCounter);

void waitCounter()
{
  sCount--;
  sCountDrawOk=1;
}
END_OF_FUNCTION(waitCounter);

/*__inline*/ void blitToScreen()
{
  blit(buffer,screen,0,0,sXoff,sYoff,280,300);
}

void restoreDisplay()
{
  dirtyScreen=1;
  redrawStatus=1;
  drawGameBorder();
  resetCounter();
}

void drawGameBorder()
{
  setGamePal();
  blit(borderBmp,screen,0,0,sXoff-20,sYoff-20,320,320);
  rect(screen,sXoff-22,sYoff-22,sXoff+300,sYoff+301,colWhite);
}

int setScreenRes(int col, int res)
{
  if(col==0)
    set_color_depth(8);

  if(col==1)
    set_color_depth(16);

  if(res==0)
  {
    if(set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0)!=0)
      return -1;
  }
  else if(res==1)
  {
    if(set_gfx_mode(GFX_AUTODETECT, 512, 384, 0, 0)!=0)
      return -1;
  }
  else if(res==2)
  {
    if(set_gfx_mode(GFX_AUTODETECT, 360, 360, 0, 0)!=0)
      return -1;
  }
  else if(res==3)
  {
    if(set_gfx_mode(GFX_AUTODETECT, 400, 300, 0, 0)!=0)
      return -1;
  }
  else if(res==4)
 {
    #ifdef DJGPP
    gOpt.vMode=0;
    if(set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0)!=0)
     return -1;
    #endif
  }
  
 
  
  else
    return -1;

  clear(screen);

  sXoff=(SCREEN_W/2)-140;
  sYoff=(SCREEN_H/2)-140;

  if(res==3)
    sYoff=20;

  set_display_switch_mode(SWITCH_AMNESIA);
  set_display_switch_callback(SWITCH_IN, restoreDisplay);
  #ifndef DJGPP
  set_window_close_hook(closeWindow);
  set_window_title("XOP 0.3");
  #endif

  set_mouse_range(sXoff-16,sYoff-16,sXoff+(280-14),sYoff+(300-14));

  colTrans=0;
  colWhite=15;
  colBlack=16;
  colMagenta=13;
  colGray=25;
  colLightGray=26;
  colBlue=32;

  palOk=1;
  if(strcmp(gfx_driver->name,"DirectDraw window")==0)
    palOk=0;
  if(strcmp(gfx_driver->name,"DirectDraw overlay")==0)
    palOk=0;

  return 0;
}

void playSample(int theSnd, int panning)
{
  if(sndPlay[theSnd]!=0 || snd[theSnd]==NULL || gOpt.soundVol==0)
    return;

  play_sample(snd[theSnd], gOpt.soundVol+(8-rand()%16), panning, 1000, FALSE);
  sndPlay[theSnd]=2;
}

int getStereoPan(int posX)
{
  return ((posX-player.x)/5.5)+128;
}

void setTrans(int stat)
{
  if(stat==0)
    drawing_mode(DRAW_MODE_SOLID,NULL,0,0);
  if(stat==1)
    drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
}

BITMAP *readPNG(char *fName)
{
  #ifndef xop_release
  return load_png(fName,pal);
  #endif

  #ifdef xop_release
  seekFile(fName);
  return load_png_file(inFile,pal);
  #endif
}

FILE *openPackedFile(char *fName)
{
  unpackFile(fName, "xop.tmp");
  return fopen("xop.tmp","r");
}

void openMOD(char *fName)
{
  if(gOpt.noMusic!=0)
    return;

  if(is_mod_playing())
    stop_mod();

  if(gMusic)
    destroy_mod(gMusic);

  if(gOpt.musicVol==0)
  {
    gMusic=NULL;
    return;
  }

  #ifndef xop_release
  gMusic=load_mod(fName);
  #endif

  #ifdef xop_release
  if(verifyFile(fName)!=0)
  {
    gMusic=NULL;
    return;
  }
  unpackFile(fName, "xop.xm");
  gMusic=load_mod("xop.xm");
  remove("xop.xm");
  #endif

  if(gMusic)
    play_mod(gMusic, TRUE);
  else
    gMusic=NULL;
}

void stopMOD()
{
  if(gOpt.noMusic!=0)
    return;

  if(is_mod_playing())
    stop_mod();

  if(gMusic)
    destroy_mod(gMusic);

  gMusic=NULL;
}

/*__inline*/ char* getPath(char *fPath)
{
  static char gPath[128];

  #ifndef xop_release
  sprintf(gPath,"data/%s/%s",gOpt.game,fPath);
  #endif

  #ifdef xop_release
  sprintf(gPath,"%s",get_filename(fPath));
  #endif

  return gPath;
}

void screenShot()
{
  static int sNum=0;
  char text[32];

  if(key[KEY_F12])
  {
    pauseStat=1;

    sprintf(text,"xop%i.png",sNum);
    while(!access(text,0))
    {
      sNum++;
      sprintf(text,"xop%i.png",sNum);
    }

    sprintf(text,"xop%i.png",sNum);
    get_pallete(pal);
    set_clip(buffer,0,0,320,320);
    blit(screen,buffer,sXoff-20,sYoff-20,0,0,320,320);
    save_png(text, buffer, pal);
    set_clip(buffer,0,0,280,300);
    sNum++;
    while(key[KEY_F12]) { }

    pauseStat=0;
  }
}

void giveFlashColor(AL_CONST PALETTE pal, int x, int y, RGB *rgb)
{
  int c;

  c=((pal[x].r+pal[x].g+pal[x].b)/3)+22;
  if(c>63)
    c=63;

  rgb->r=c;
  rgb->g=c;
  rgb->b=c;
}

void giveFlashColorRed(AL_CONST PALETTE pal, int x, int y, RGB *rgb)
{
  int c;

  c=pal[x].r+20;
  if(c>63)
    c=63;

  rgb->r=c;
  rgb->g=0;
  rgb->b=0;
}

void setGamePal()
{
  int x;

  sprPlay[0]=readPNG(getPath("png/system/pal01.png"));
  destroy_bitmap(sprPlay[0]);
  sprPlay[0]=NULL;

  pal[0].r=0; pal[0].g=0; pal[0].b=0;

  for(x=0; x<256; x++)
  {
    palGame[x].r=pal[x].r;
    palGame[x].g=pal[x].g;
    palGame[x].b=pal[x].b;
  }

  set_pallete(palGame);
}

int getColor(int theCol)
{
  return theCol;

  return 0;
}

int makeRGBfromPal(int palCol)
{
  return makecol(palGame[palCol].r*4,palGame[palCol].g*4,palGame[palCol].b*4);
}

void putpixel8(BITMAP *theBmp, int x, int y, int c)
{
  _putpixel(theBmp,x,y,c);
}

int getpixel8(BITMAP *theBmp, int x, int y)
{
  return _getpixel(theBmp,x,y);
}

void bombOut(int error, int num)
{
  char eString[256];

  remove("xop.tmp");

  #ifndef xop_release // missing file stuff, not needed for release (handled by bombOutText)
  if(error==0) sprintf(eString,"No player def file found");
  if(error==1) sprintf(eString,"No script file found");
  if(error==2) sprintf(eString,"No map file found");
  if(error==3) sprintf(eString,"No act file found");
  if(error==4) sprintf(eString,"No tile list found/missing sprite.dat/missing sound.dat");
  if(error==5) sprintf(eString,"tile png file %i is missing in level.til",num);
  if(error==6) sprintf(eString,"sprite png file %i is missing or invalid in sprite.dat",num);
  if(error==9) sprintf(eString,"No thing def file found");
  if(error==10) sprintf(eString,"No shrap def file found");
  if(error==11) sprintf(eString,"No shot def file found");
  if(error==12) sprintf(eString,"No move def file found");
  if(error==14) sprintf(eString,"No circle def file found");
  if(error==15) sprintf(eString,"No explosion def file found");
  if(error==16) sprintf(eString,"No link def file found");
  if(error==19) sprintf(eString,"wav file %i is missing or invalid in sound.dat",num);
  if(error==21) sprintf(eString,"Unable to find data directory");
  if(error==22) sprintf(eString,"sprite png file %i is missing or invalid in level.spr",num);
  if(error==33) sprintf(eString,"No level description file found");
  #endif

  if(error==7) sprintf(eString,"Unable to init sound : %s. Try -nosound", allegro_error);
  if(error==8) sprintf(eString,"Unable to init graphics mode : %s", allegro_error);

  if(error==13) sprintf(eString,"%i",num); // unused

  if(error==17) sprintf(eString,"Unable to allocate space for map/script data, requested amount was %i bytes",num);
  if(error==18) sprintf(eString,"Unable to init mod music, try -nomusic");

  if(error==20) sprintf(eString,"Unable to save options");

  if(error==23) sprintf(eString,"Duplicate sprite ID %i",num);
  if(error==24) sprintf(eString,"Couldn't write temp data file");
  if(error==25) sprintf(eString,"Unable to create highscore file");
  if(error==26) sprintf(eString,"Invalid level %i",num);
  if(error==27) sprintf(eString,"Sprite limit reached (max 512)");
  if(error==28) sprintf(eString,"Thing %i doesn't have a matching entry in tact.def/tdef.def",num);
  if(error==29) sprintf(eString,"Not enough things for chained thing"); // unused
  if(error==30) sprintf(eString,"Script number invalid on line %i",num);
  if(error==31) sprintf(eString,"Attempted to create a chain/snake link without a followup link");
  if(error==32) sprintf(eString,"Attempted to use datObj number %i, which was marked as not loaded",num);

  set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);

  if(error<=33)
    allegro_message("Ack!  Error %i: %s\n",error,eString);
  else
    allegro_message("Ack!  Error %i: Unknown Error\n",error);

  doQuit();
}

void bombOutText(int error, char *text)
{
  char eString[256];

  remove("xop.tmp");

  if(error==0) sprintf(eString,"Couldn't find file '%s' in datafile",text);
  if(error==1) sprintf(eString,"Error reading file '%s' from datafile",text);
  if(error==2) sprintf(eString,"Unable to open datafile '%s'",text);
  if(error==3) sprintf(eString,"File name '%s' is too long (max 24 chars)",text);

  set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);

  if(error<=3)
    allegro_message("Ack!  Error %i: %s",error+200,eString);
  else
    allegro_message("Ack!  Error %i: Unknown Error",error+200);

  doQuit();
}