#ifndef xop_h_thing
#define xop_h_thing

/*

big description of all tObj variables

movement

x=x position
y=y position
z=z position
sx=x speed, added to x every game loop
sy=y speed, added to y every game loop
sz=z speed, added to z every game loop
gx=x gravity, added to sx every game loop
gy=y gravity, added to sy every game loop
gz=z gravity, added to sz every game loop
tx=x master link position, used for link type 2
ty=y master link position, used for link type 2
tz=z master link position, used for link type 2
slimit=speed limit, clips sx, sy and sz to it's value in negative or positive
rot=rotation
srot=rotation speed
grot=rotation gravity

shield

shield=the 'hit points' of the thing.  when this is <=0, the thing is killed
sMax=max amount of shield, given when the thing is created

hit rectangle

offX=sprite offset x
offY=sprite offset y
hrX1=hit rectangle x 1, the upper left coordinate
hrY1=hit rectangle y 1, the upper left coordinate
hrX2=hit rectangle x 2, the lower right coordinate
hrY2=hit rectangle y 2, the lower right coordinate
hrX3=hrX2-hrX1, x distance between the two coordinates
hrY3=hrY2-hrY1, y distance between the two coordinates
hrZ2=hit rectangle z, the z 'depth'
w2=sprMain[thing[x].sprite]->w2/2; half the distance of the sprite x
h2=sprMain[thing[x].sprite]->h2/2; half the distance of the sprite y

stat=the thing's status.  valid values:

STAT_NULL    -1 : thing is not currently being used, will be picked up by findEmptyT() and used as a new thing
STAT_NORMAL   0 : normal status
STAT_DYING    1 : thing is dying, all hit detection is disabled
STAT_SHOOTING 2 : thing is shooting, same as normal
STAT_HIT      3 : thing is hit, same as normal

type=thing type, refers to thingDef[] and thingAct[]
sprite=number that refers to a array in sprMain[maxsprite]; used when drawing the thing in drawScreen()

timers

dTimer=death timer, counts down to 0, then the thing is killed

timer=action table timer, is incremented every game loop by 1
tlimit=action table timer limit
act=number that refers to an array in actMain[maxact];
next=number that refers to an array in actMain[maxact]; the thing's act will be set to this when timer reaches tlimit (using copyActT())

sTimer=shot timer, is incremented every game loop by 1
sTimerLimit=shot timer limit
sNext=number that refers to an array in shotDef[maxdat]; the thing will shoot this shotDef when sTimer reaches sTimerLimit (using doThingShoot())
sCutoff=decremented every time the thing shoots - when this is 0, the shot pattern will end
sCutoffNext=the next shotDef after cutoff reaches 0
sWait=wait between shots
sType=starting shotDef
sRotInc=shot rotation increment
sSpdInc=shot speed increment

amTimer=automovement timer
amTimerLimit=automovement timer limit
amNext=next automovement def

links

tLink=number that refers to another thing in thing[maxthing]; this is the master link
tLink2=number that refers to another thing in thing[maxthing]; this is the secondary link
tLink3=number that refers to another thing in thing[maxthing]; this is the third link
tLinkNum=number that refers to an array in linkDef; used by a thing that has links attached to it
tLinkNumDam=number that refers to an array in linkDef; used by a thing that will have visible damage things attached to it
tLinkSrc=number that refers to an array in linkDef; used by a thing that is linked
tLinkType=link type
tLinkPos=position in the link

misc.

drawShadow=refers to the background the thing's shadow is to be drawn on.  if the thing doesn't draw a shadow, this is set to -1
hitPshot=if 1, thing can be hit by player shots
groundHit=ground hit detect/explosion type for when thing collides with a background
deathscr=script to run on death
madeDamage=used to determine how much the thing is damaged, and how much visible damage has been placed onto it
tid=thing id, used for various purposes
dirRot=if 1, thing's speed will be 'rotated' to the thing's rotation
sprRot=if 1, thing's sprite will be drawn without rotation
rangeAMT=automovement type to set thing to when player is within range pixels
range=range for rangeAMT activation
tChase=used to identify a thing to be followed, set in copyAMT()
dir=thing direction; 0=left, 1=right; has the effect of reversing sx and making shots fire from the opposite side of the thing
flash=if 1, thing will be flashing
flashCount=incremented by one every game loop; used for making the thing flash when damaged
zScale=amount to add to z to scale the sprite

circ[]=each one refers to a array in circ[maxcirc];

*/

typedef struct tObj
{
    float x,y,z,sx,sy,sz,gx,gy,gz,tx,ty,tz,slimit,rot,srot,grot,shield,sMax; // position, speed, gravity, shield

    int stat,type,sprite;
    int offX,offY,hrX1,hrY1,hrX2,hrY2,hrX3,hrY3,hrZ2,w2,h2;     // hit rectangle
    int dTimer,timer,tlimit,act,next;            // action timer
    int sTimer,sTimerLimit,sNext,sCutoff,sCutoffNext,sWait,sType,sRotInc,sSpdInc; // shot timer
    int amTimer,amTimerLimit,amNext;                  // automovement timer
    int tLink,tLink2,tLink3,tLinkNum,tLinkNumDam,tLinkSrc,tLinkType,tLinkPos; // links

    int drawShadow,hitPshot,groundHit,deathscr,madeDamage,tid,dirRot,sprRot,rangeAMT,range,tChase;
    int dir,flash,flashCount,zScale,circ[maxtcirc];
} tObj;

typedef struct timerObj
{
  int timer;
  int scriptAct;
  int x,y,z,tLookup;
} timerObj;

typedef struct sortObj
{
  int key,num;
} sortObj;

//struct tObj thing[maxthing];
tObj *thing;
struct timerObj timer[maxtimer];
struct sortObj oSort[maxthing];

void initThing(int tNum, 
               float ix, float iy, float iz,float isx, float isy, float isz,
               int idir, int itype, int istat, int iact, float ishield);
void doThings();
void doThingTimers(int tNum);
void doThingMovement(int tNum);
void doThingLinks(int tNum);
void doThingHit(int tNum);

void doThingPowerup(int tNum, int pType);

void sortThings();
void doThingShoot(int tNum, int sNum);
void killThing(int tNum);          // kill a thing, make explosions, etc
void clearThing(int tNum);         // clear a thing with no effects
void copyActT(int tNum, int aNum); // copy action
void copyAMT(int tNum, int amNum); // copy automovement type
int findEmptyT();                  // returns a thing with STAT_NULL (-1)
int findEmptyTOffset(int offset);      // returns a thing with STAT_NULL (-1) and higher than mNum

#endif